@@ -13,7 +13,7 @@ while [ "$#" -gt 0 ]; do
       OPT_SHARED=1
       ;;
     *)
-      [ -x "$1" -a -z "$rhash" -a -x "$1" ] && rhash="$(cd $(dirname $1) && echo $PWD/${1##*/})"
+      test -z "$rhash" && rhash="$1"
       ;;
   esac
   shift
@@ -22,14 +22,26 @@ _sdir="$(dirname "$0")"
 SCRIPT_DIR="$(cd "$_sdir" && pwd)"
 UPPER_DIR="$(cd "$_sdir/.." && pwd)"
 
-# detect binary if not specified by command line
-[ -x "$rhash" ] || rhash="$UPPER_DIR/rhash";
-[ -x "$rhash" ] || rhash="`which rhash`"
-if [ ! -x "$rhash" ]; then
-  echo "Fatal: $rhash not found"
+# find the path of rhash binary
+if test -x "$rhash"; then
+  rhash="$(cd $(dirname $rhash) && echo $PWD/${rhash##*/})"
+elif test -z "$rhash"; then
+  command -v rhash 2>/dev/null >/dev/null && _path="$(command -v rhash 2>/dev/null)"
+  if test -x "$_path"; then
+    rhash="$_path"
+  elif test -x /usr/bin/rhash; then
+    rhash=/usr/bin/rhash
+  elif test -x /usr/local/bin/rhash; then
+    rhash=/usr/local/bin/rhash
+  fi
+fi
+if [ ! -f "$rhash" ]; then
+  echo "Fatal: file $rhash not found"
+  exit 1
+elif [ ! -x "$rhash" ]; then
+  echo "Fatal: $rhash is not an executable file"
   exit 1
 fi
-echo "Testing $rhash"
 
 win32()
 {
@@ -74,7 +86,7 @@ for _tmp in "$TMPDIR" "$TEMPDIR" "/tmp" ; do
   [ -d "$_tmp" ] && break
 done
 RANDNUM=$RANDOM
-[ -z $RANDNUM ] && which jot >/dev/null && RANDNUM=$(jot -r 1 1 32767)
+test -z "$RANDNUM" && jot -r 1 2>/dev/null >dev/null && RANDNUM=$(jot -r 1 1 32767) || RANDNUM=0
 RHASH_TMP="$_tmp/rhash-test-$RANDNUM-$$"
 remove_tmpdir()
 {
@@ -181,21 +193,26 @@ check "$TEST_RESULT" "d41d8cd98f00b204e9800998ecf8427e" .
 TEST_RESULT=$( printf "" | $rhash -p "%m" - )
 check "$TEST_RESULT" "d41d8cd98f00b204e9800998ecf8427e" .
 # test verification of empty file
-TEST_RESULT=$( $rhash -c "$EMPTY_FILE" | grep "^[^-]" )
+TEST_RESULT=$( $rhash -c --brief "$EMPTY_FILE" | tr -d '\r' )
 check "$TEST_RESULT" "Nothing to verify"
-rm "$EMPTY_FILE"
 
-# Test the SFV format using test1K.data from the previous test
+# Test the SFV format using test1K.data
 new_test "test default format:        "
 MATCH_LOG="$RHASH_TMP/match_err.log"
-$rhash test1K.data | (
+$rhash test1K.data | tr -d '\r' | (
   read l; match_line "$l" "^; Generated by RHash"
   read l; match_line "$l" "^; Written by"
   read l; match_line "$l" "^;\$"
   read l; match_line "$l" "^; *1024  [0-9:\.]\{8\} [0-9-]\{10\} test1K.data\$"
   read l; match_line "$l" "^test1K.data B70B4C26\$"
 ) > "$MATCH_LOG"
-[ ! -s "$MATCH_LOG" ] && echo "Ok" || echo "Failed" && cat "$MATCH_LOG"
+if [ ! -s "$MATCH_LOG" ]; then
+  echo "Ok"
+else
+  echo "Failed"
+  fail_cnt=$((fail_cnt+1))
+  cat "$MATCH_LOG"
+fi
 rm -f "$MATCH_LOG"
 
 new_test "test %x, %b, %B modifiers:  "
@@ -245,9 +262,19 @@ fi
 
 new_test "test checking all hashes:   "
 TEST_RESULT=$( $rhash --simple -a test1K.data | $rhash -vc - 2>/dev/null | grep test1K.data )
+match "$TEST_RESULT" "^test1K.data *OK" .
+# verify a filepath started by star ('*') character
+TEST_RESULT=$( echo "b70b4c26 *test1K.data" | $rhash -Cc - 2>/dev/null | grep test1K.data )
 match "$TEST_RESULT" "^test1K.data *OK"
 
-new_test "test checking magnet link:  "
+new_test "test magnet links:          "
+TEST_RESULT=$( $rhash --magnet --crc32c test1K.data )
+TEST_EXPECTED="magnet:?xl=1024&dn=test1K.data&xt=urn:crc32c:2cdf6e8f"
+check "$TEST_RESULT" "$TEST_EXPECTED" .
+# test magnet default format
+TEST_RESULT=$( $rhash --magnet test1K.data )
+TEST_EXPECTED="magnet:?xl=1024&dn=test1K.data&xt=urn:tree:tiger:4oqy25un2xhidqpv5u6bxaz47inucygibk7lfni&xt=urn:ed2k:5ae257c47e9be1243ee32aabe408fb6b&xt=urn:aich:lmagnhcibvop7pp2rpn2tflbcyhs2g3x"
+check "$TEST_RESULT" "$TEST_EXPECTED" .
 # also test that '--check' verifies files in the current directory
 mkdir magnet_dir && $rhash --magnet -a test1K.data > magnet_dir/t.magnet
 TEST_RESULT=$( $rhash -vc magnet_dir/t.magnet 2>&1 | grep test1K.data )
@@ -255,13 +282,13 @@ TEST_EXPECTED="^test1K.data *OK"
 match "$TEST_RESULT" "$TEST_EXPECTED"
 
 new_test "test bsd format checking:   "
-TEST_RESULT=$( $rhash --bsd -a test1K.data | $rhash -c --skip-ok - 2>&1 | grep -v '^--' | grep -v '^$' )
+TEST_RESULT=$( $rhash --bsd -a test1K.data | $rhash -c --skip-ok --brief - 2>&1 )
 check "$TEST_RESULT" "Everything OK"
 
 new_test "test checking w/o filename: "
-$rhash -p '%c\n%m\n%e\n%h\n%g\n%t\n%a\n' test1K.data > t.sum
-TEST_RESULT=$( $rhash -vc t.sum 2>&1 | grep '^test1K.data' | grep -v ' OK *$' )
-check "$TEST_RESULT" ""
+$rhash -p '%c\n%m\n%e\n%h\n%g\n%t\n%a\n' test1K.data > test1K.data.sum
+TEST_RESULT=$( $rhash -c --brief test1K.data.sum 2>&1 | grep -v '^test1K.data.*OK' )
+check "$TEST_RESULT" "Everything OK"
 
 new_test "test checking embedded crc: "
 printf 'A' > 'test_[D3D99E8B].data' && printf 'A' > 'test_[D3D99E8C].data'
@@ -333,11 +360,17 @@ $rhash -H none-existent.file 2>/dev/null
 check "$?" "1" .
 $rhash -c none-existent.file 2>/dev/null
 check "$?" "1" .
+A_SFV="$RHASH_TMP/a.sfv"
+printf "00000000 none-existent.file\\n00000000 test-empty.file\\n" > "$A_SFV"
+$rhash -c "$A_SFV" >/dev/null
+check "$?" "1" .
+$rhash -c --ignore-missing "$A_SFV" >/dev/null
+check "$?" "0" .
 $rhash -H test1K.data >/dev/null
 check "$?" "0"
 UNWRITABLE_FILE="$RHASH_TMP/test-unwritable.file"
 printf "" > "$UNWRITABLE_FILE" && chmod a-w "$UNWRITABLE_FILE"
-# check if really unwritable, since superuser still can write
+# check if the file is really unwritable, since the superuser still can write to it
 if ! test -w "$UNWRITABLE_FILE" ; then
  $rhash -o "$UNWRITABLE_FILE" -H test1K.data 2>/dev/null
  check "$?" "2" .
