@@ -1,5 +1,8 @@
 /* win_utils.c - utility functions for Windows and CygWin */
 #if defined(_WIN32) || defined(__CYGWIN__)
+
+/* Fix #138: require to use MSVCRT implementation of *printf functions */
+#define __USE_MINGW_ANSI_STDIO 0
 #include "win_utils.h"
 #include <windows.h>
 
@@ -416,7 +419,7 @@ static int is_directory(const char* path)
 }
 
 /**
- * Set the locale directory relative to ${PROGRAM_DIR}/LOCALEDIR.
+ * Set the locale directory relative to ${PROGRAM_DIR}/locale.
  */
 void setup_locale_dir(void)
 {
@@ -462,24 +465,27 @@ void setup_locale_dir(void)
  */
 static int win_vfprintf_encoded(FILE* out, const char* format, int str_type, va_list args)
 {
+	int res;
 	if (!win_is_console_stream(out)) {
 		return vfprintf(out, (format ? format : "%s"), args);
 	} else if (str_type == USE_CSTR_ARGS) {
 		/* thread-unsafe code: using a static buffer */
 		static char buffer[8192];
-		int res = vsnprintf(buffer, sizeof(buffer) - 1, format, args);
+		res = vsnprintf(buffer, sizeof(buffer) - 1, format, args);
 		if (res >= 0) {
 			wchar_t *wstr = cstr_to_wchar_buffer(buffer, console_data.primary_codepage, console_data.printf_result, sizeof(console_data.printf_result));
 			res = (wstr ? fwprintf(out, L"%s", wstr) : -1);
 		}
-		return res;
 	} else {
 		wchar_t* wformat = (!format || (format[0] == '%' && format[1] == 's' && !format[2]) ? L"%s" :
 			cstr_to_wchar_buffer(format, console_data.primary_codepage, console_data.format_buffer, sizeof(console_data.format_buffer)));
-		int res = vfwprintf(out, (wformat ? wformat : L"[UTF8 conversion error]\n"), args);
+		res = vfwprintf(out, (wformat ? wformat : L"[UTF8 conversion error]\n"), args);
 		assert(str_type == USE_WSTR_ARGS);
-		return res;
 	}
+	/* fix: win7 incorrectly sets _IOERR(=0x20) flag of the stream for UTF-8 encoding, so clear it */
+	if (res >= 0)
+		clearerr(out);
+	return res;
 }
 
 /**
@@ -543,8 +549,7 @@ int win_fprintf_warg(FILE* out, const char* format, ...)
  */
 size_t win_fwrite(const void* ptr, size_t size, size_t count, FILE* out)
 {
-	if ((out != stdout || !(console_data.console_flags & 1))
-		&& (out != stderr || !(console_data.console_flags & 2)))
+	if (!win_is_console_stream(out))
 		return fwrite(ptr, size, count, out);
 	{
 		size_t i;
@@ -556,18 +561,27 @@ size_t win_fwrite(const void* ptr, size_t size, size_t count, FILE* out)
 		if (i == size)
 		{
 			wchar_t* wstr = rsh_malloc(sizeof(wchar_t) * (size + 1));
+			int res;
 			for (i = 0; i < size; i++)
 				wstr[i] = (wchar_t)buf[i];
-			wstr[size] = 0;
-			fwprintf(out, L"%s", wstr);
+			wstr[size] = L'\0';
+			res = fwprintf(out, L"%s", wstr);
 			free(wstr);
-			return count;
+			if (res < 0)
+				return res;
 		}
-		for (i = 0; (i + 8) <= size; i += 8)
-			fwprintf(out, L"%C%C%C%C%C%C%C%C", buf[i], buf[i + 1], buf[i + 2],
-				buf[i + 3], buf[i + 4], buf[i + 5], buf[i + 6], buf[i + 7]);
-		for (; i < size; i++)
-			fwprintf(out, L"%C", buf[i]);
+		else
+		{
+			for (i = 0; (i + 8) <= size; i += 8)
+				if (fwprintf(out, L"%C%C%C%C%C%C%C%C", buf[i], buf[i + 1], buf[i + 2],
+						buf[i + 3], buf[i + 4], buf[i + 5], buf[i + 6], buf[i + 7]) < 0)
+					return -1;
+			for (; i < size; i++)
+				if (fwprintf(out, L"%C", buf[i]) < 0)
+					return -1;
+		}
+		/* fix: win7 incorrectly sets _IOERR(=0x20) flag of the stream for UTF-8 encoding, so clear it */
+		clearerr(out);
 		return count;
 	}
 }
